@@ -5,5 +5,5 @@
 # See: https://packaging.python.org/guides/single-sourcing-package-version
 
 __author__ = """David Huard"""
-__email__ = 'huard.david@ouranos.ca'
-__version__ = '0.10.0'
+__email__ = "huard.david@ouranos.ca"
+__version__ = "0.1.7"
