@@ -1,15 +1,14 @@
 """
-Raven model definition
+Base classes
+------------
 
-Classes
--------
-
-Raven : A generic class that knows how to launch the model from completed rv files.
-Ostrich:
+The `Raven` class is the base class that implements model setup, execution and output retrieval, while the `Ostrich`
+class is the base class adapting `Raven` to work with the Ostrich calibration tool.
 
 """
 import csv
 import datetime as dt
+import operator
 import os
 import shutil
 import stat
@@ -24,7 +23,19 @@ import xarray as xr
 
 import ravenpy
 
-from .rv import RV, RVI, Ost, RavenNcData, RVFile, isinstance_namedtuple, parse_solution
+from .rv import (
+    RV,
+    RVI,
+    Ost,
+    RavenNcData,
+    RVFile,
+    get_states,
+    isinstance_namedtuple,
+    parse_solution,
+)
+
+RAVEN_EXEC_PATH = os.getenv("RAVENPY_RAVEN_BINARY_PATH") or shutil.which("raven")
+OSTRICH_EXEC_PATH = os.getenv("RAVENPY_OSTRICH_BINARY_PATH") or shutil.which("ostrich")
 
 
 class Raven:
@@ -33,8 +44,8 @@ class Raven:
     This class is used to run the RAVEN model from user-provided configuration files. It can also be subclassed with
     configuration templates for emulated models, allowing direct calls to the models.
 
-    Usage
-    -----
+    Examples
+    --------
     >>> r = Raven('/tmp/testdir')
     >>> r.configure()
 
@@ -66,18 +77,6 @@ class Raven:
         ],
     }
 
-    # Expected units (pint-compatible)
-    _units = {
-        "tasmin": "degC",
-        "tasmax": "degC",
-        "tas": "degC",
-        "pr": "mm/d",
-        "rainfall": "mm/d",
-        "prsn": "mm/d",
-        "evspsbl": "mm/d",
-        "water_volume_transport_in_river_channel": "m**3/s",
-    }
-
     _parallel_parameters = [
         "params",
         "hru_state",
@@ -100,6 +99,20 @@ class Raven:
         workdir : str, Path
           Directory for the model configuration and outputs. If None, a temporary directory will be created.
         """
+
+        if not RAVEN_EXEC_PATH:
+            raise RuntimeError(
+                "Could not find raven binary in PATH, and RAVENPY_RAVEN_BINARY_PATH env variable is not set"
+            )
+
+        if not OSTRICH_EXEC_PATH:
+            raise RuntimeError(
+                "Could not find ostrich binary in PATH, and RAVENPY_OSTRICH_BINARY_PATH env variable is not set"
+            )
+
+        self.raven_exec = RAVEN_EXEC_PATH
+        self.ostrich_exec = OSTRICH_EXEC_PATH
+
         workdir = workdir or tempfile.mkdtemp()
         self._rvs = []
 
@@ -114,9 +127,7 @@ class Raven:
         self.ind_outputs = {}  # Individual files for all simulations
         self.outputs = {}  # Aggregated files
         self.singularity = False  # Set to True to launch Raven with singularity.
-        self.raven_exec = ravenpy.raven_exec
-        self.raven_simg = ravenpy.raven_simg
-        self.ostrich_exec = ravenpy.ostrich_exec
+        self.raven_simg = None  # ravenpy.raven_simg
         self._name = None
         self._defaults = {}
         self.rvfiles = {}
@@ -137,7 +148,7 @@ class Raven:
         self.exec_path = self.workdir / "exec"
         self.final_path = self.workdir / self.final_dir
         self._psim = 0
-        self._pdim = None  # Parallel dimension (either params or nbasins)
+        self._pdim = None  # Parallel dimension (either initparam, params or region)
 
     @property
     def output_path(self):
@@ -257,6 +268,7 @@ class Raven:
 
     def assign(self, key, value):
         """Assign parameter to rv object that has a key with the same name."""
+
         assigned = False
         for ext, obj in self.rvobjs.items():
             if hasattr(obj, key):
@@ -378,8 +390,8 @@ class Raven:
         launch the Raven executable. If the configuration files are templates, values can be formatted by passing
         dictionaries keyed by their extension.
 
-        Example
-        -------
+        Examples
+        --------
         >>> r = Raven()
         >>> r.configure(rvi='path to template', rvp='...'}
         >>> r.run(ts, start_date=dt.datetime(2000, 1, 1), area=1000, X1=67)
@@ -395,15 +407,26 @@ class Raven:
         for p in self._parallel_parameters:
             a = kwds.pop(p, None)
 
-            if p in ["params", "hrus", "basin_state", "hru_state"] and a is not None:
+            if a is not None and p in ["params", "basin_state", "hru_state"]:
                 pdict[p] = np.atleast_2d(a)
             else:
                 pdict[p] = np.atleast_1d(a)
 
-        # Number of parallel loops is dictated by the number of parameters or nc_index.
-        nloops = max(len(pdict["params"]), len(pdict["nc_index"]))
+        # Number of parallel loops is dictated by the number of parallel parameters or nc_index.
+        plen = {pp: len(pdict[pp]) for pp in self._parallel_parameters + ["nc_index"]}
+
+        # Find the longest parallel array and its length
+        longer, nloops = max(plen.items(), key=operator.itemgetter(1))
+
+        # Assign the name of the parallel dimension
+        # nbasins is set by RavenC++
         if nloops > 1:
-            self._pdim = "nbasins" if len(pdict["nc_index"]) > 1 else "params"
+            self._pdim = {
+                "params": "params",
+                "hru_state": "state",
+                "basin_state": "state",
+                "nc_index": "nbasins",
+            }[longer]
 
         for key, val in pdict.items():
             if len(val) not in [1, nloops]:
@@ -438,7 +461,7 @@ class Raven:
             self.handle_date_defaults(ts)
             self.set_calendar(ts)
 
-        # Loop over parallel parameters
+        # Loop over parallel parameters - sets self.rvi.run_index
         procs = []
         for self.psim in range(nloops):
             for key, val in pdict.items():
@@ -484,29 +507,25 @@ class Raven:
         ----------
         solution : str, Path
           Path to solution file. If None, will use solution from last model run if any.
-
-        Note that the starting date of the next run should be identical to the end date of the previous run.
         """
         if solution is None:
             fn = self.outputs["solution"]
         else:
             fn = solution
 
-        rvc = RVFile(fn)
-        rvc.rename(self.name)
-        self.rvfiles["rvc"] = rvc
+        self.rvc.parse(Path(fn).read_text())
 
-    def parse_results(self, path=None):
+    def parse_results(self, path=None, run_name=None):
         """Store output files in the self.outputs dictionary."""
         # Output files default names. The actual output file names will be composed of the run_name and the default
         # name.
         path = path or self.exec_path
-
+        run_name = run_name or getattr(self.rvi, "run_name", "")
         patterns = {
-            "hydrograph": "*Hydrographs.nc",
-            "storage": "*WatershedStorage.nc",
-            "solution": "*solution.rvc",
-            "diagnostics": "*Diagnostics.csv",
+            "hydrograph": f"{run_name}*Hydrographs.nc",
+            "storage": f"{run_name}*WatershedStorage.nc",
+            "solution": f"{run_name}*solution.rvc",
+            "diagnostics": f"{run_name}*Diagnostics.csv",
         }
 
         for key, pattern in patterns.items():
@@ -540,7 +559,7 @@ class Raven:
             ds = [xr.open_dataset(fn) for fn in files]
             try:
                 # We aggregate along the pdim dimensions.
-                out = xr.concat(ds, self._pdim, data_vars="different")
+                out = xr.concat(ds, self._pdim, data_vars="all")
                 out.to_netcdf(outfn)
                 return outfn
             except (ValueError, KeyError):
@@ -658,9 +677,8 @@ class Raven:
         if rvi.start_date in [None, dt.datetime(1, 1, 1)]:
             rvi.start_date = start
 
-        else:
-            if rvi.end_date in [None, dt.datetime(1, 1, 1)]:
-                rvi.end_date = end
+        if rvi.end_date in [None, dt.datetime(1, 1, 1)]:
+            rvi.end_date = end
 
     @property
     def rvs(self):
@@ -705,6 +723,26 @@ class Raven:
     def solution(self):
         if self.outputs["solution"].suffix == ".rvc":
             return parse_solution(self.outputs["solution"].read_text())
+        elif self.outputs["solution"].suffix == ".zip":
+            return [
+                parse_solution(fn.read_text()) for fn in self.ind_outputs["solution"]
+            ]
+
+    def get_final_state(self, hru_index=1, basin_index=1):
+        """Return model state at the end of simulation.
+
+        Parameters
+        ----------
+        hru_index : None, int
+          Set index value or None to get all HRUs.
+        basin_index : None, int
+          Set index value or None to get all basin states.
+        """
+        solution = self.solution
+        if isinstance(solution, dict):
+            return get_states(solution, hru_index, basin_index)
+        else:
+            return zip(*[get_states(s, hru_index, basin_index) for s in solution])
 
     @property
     def diagnostics(self):
@@ -773,11 +811,10 @@ class Ostrich(Raven):
     """Wrapper for OSTRICH calibration of RAVEN hydrological model
 
     This class is used to calibrate RAVEN model using OSTRICH from user-provided configuration files. It can also be
-    subclassed with
-    configuration templates for emulated models, allowing direct calls to the models.
+    subclassed with configuration templates for emulated models, allowing direct calls to the models.
 
-    Usage
-    -----
+    Examples
+    --------
     >>> r = Ostrich('/tmp/testdir')
     >>> r.configure()
 
@@ -914,8 +951,8 @@ class Ostrich(Raven):
     def ost2raven(self, ops):
         """Return model parameters.
 
-        Note
-        ----
+        Notes
+        -----
         This method should be subclassed by emulators for which Ostrich has different parameters than the original
         Raven model.
         """
