@@ -219,78 +219,82 @@ class RoutingProductGridWeightImporter:
 
     CRS_LLDEG = 4326  # EPSG id of lat/lon (deg) coordinate referenence system (CRS)
     CRS_CAEA = 3573  # EPSG id of equal-area    coordinate referenence system (CRS)
-    HRU_ID_FIELD = "HRU_ID"
+    ROUTING_ID_FIELD = "HRU_ID"
+    NETCDF_INPUT_FIELD = "NetCDF_col"
     DIM_NAMES = ("lon", "lat")
     VAR_NAMES = ("lon", "lat")
+    AREA_ERROR_THRESHOLD = 0.05
 
     def __init__(
         self,
-        shapefile_path,
-        nc_data_path,
+        input_file_path,
+        routing_file_path,
         dim_names=DIM_NAMES,
         var_names=VAR_NAMES,
-        hru_id_field=HRU_ID_FIELD,
+        routing_id_field=ROUTING_ID_FIELD,
+        netcdf_input_field=NETCDF_INPUT_FIELD,
         gauge_ids=None,
         sub_ids=None,
+        area_error_threshold=AREA_ERROR_THRESHOLD,
     ):
         self._dim_names = tuple(dim_names)
         self._var_names = tuple(var_names)
-        self._hru_id_field = hru_id_field
+        self._routing_id_field = routing_id_field
+        self._netcdf_input_field = netcdf_input_field
         self._gauge_ids = gauge_ids or []
         self._sub_ids = sub_ids or []
+        self._area_error_threshold = area_error_threshold
 
         assert not (
             self._gauge_ids and self._sub_ids
         ), "Only one of gauge_ids or sub_ids can be specified"
 
-        if Path(shapefile_path).suffix == ".zip":
-            shapefile_path = f"zip://{shapefile_path}"
-        self._shapes = geopandas.read_file(shapefile_path)
-        # Note that we cannot use xarray because it complains about variables and dimensions
-        # having the same name.
-        self._data = nc4.Dataset(nc_data_path)
+        self._input_is_netcdf = True
+
+        input_file_path = Path(input_file_path)
+        if input_file_path.suffix == ".nc":
+            # Note that we cannot use xarray because it complains about variables and dimensions
+            # having the same name.
+            self._input_data = nc4.Dataset(input_file_path)
+        elif input_file_path.suffix == ".zip":
+            self._input_data = geopandas.read_file(f"zip://{input_file_path}")
+            self._input_is_netcdf = False
+        elif input_file_path.suffix == ".shp":
+            self._input_data = geopandas.read_file(input_file_path)
+            self._input_is_netcdf = False
+        else:
+            raise ValueError(
+                "The input file must be a shapefile (.shp or .zip) or NetCDF"
+            )
+
+        routing_file_path = Path(routing_file_path)
+        if routing_file_path.suffix == ".zip":
+            routing_file_path = f"zip://{routing_file_path}"
+        self._routing_data = geopandas.read_file(routing_file_path)
 
     def extract(self) -> GridWeightsCommand:
-        # Raven numbering is:
-        #
-        #      [      1      2      3   ...     1*nlon
-        #        nlon+1 nlon+2 nlon+3   ...     2*nlon
-        #           ...    ...    ...   ...     ...
-        #           ...    ...    ...   ...  nlat*nlon ]
-        #
-        # --> Making sure shape of lat/lon fields is like that
-        #
-        lon_var = self._data.variables[self._var_names[0]][:]
-        if self._data.variables[self._var_names[0]].dimensions == self._dim_names:
-            lon_var = np.transpose(lon_var)
 
-        lat_var = self._data.variables[self._var_names[1]][:]
-        if self._data.variables[self._var_names[1]].dimensions == self._dim_names:
-            lat_var = np.transpose(lat_var)
+        self._prepare_input_data()
 
-        lath, lonh = self._create_gridcells_from_centers(lat_var, lon_var)
+        # Read routing data
 
-        nlon = np.shape(lon_var)[1]
-        nlat = np.shape(lat_var)[0]
-
-        # shape     = shape.to_crs(epsg=crs_lldeg)        # this is lat/lon in degree
         # WGS 84 / North Pole LAEA Canada
-        self._shapes = self._shapes.to_crs(
+        self._routing_data = self._routing_data.to_crs(
             epsg=RoutingProductGridWeightImporter.CRS_CAEA
         )
 
-        self._shapes = self._shapes.drop_duplicates(self._hru_id_field).sort_values(
-            self._hru_id_field
-        )
+        self._routing_data = self._routing_data.drop_duplicates(
+            self._routing_id_field
+        ).sort_values(self._routing_id_field)
 
         # Make sure those are ints
-        self._shapes.SubId = self._shapes.SubId.astype(int)
-        self._shapes.DowSubId = self._shapes.DowSubId.astype(int)
+        self._routing_data.SubId = self._routing_data.SubId.astype(int)
+        self._routing_data.DowSubId = self._routing_data.DowSubId.astype(int)
 
         if self._gauge_ids:
             # Extract the SubIDs of the gauges that were specified at input
             self._sub_ids = (
-                self._shapes.loc[self._shapes.Obs_NM.isin(self._gauge_ids)]
+                self._routing_data.loc[self._routing_data.Obs_NM.isin(self._gauge_ids)]
                 .SubId.unique()
                 .tolist()
             )
@@ -304,7 +308,7 @@ class RoutingProductGridWeightImporter:
             # starting from the list supplied by the user (either directly, or via their gauge IDs).. We first
             # build a map of downSubID -> subID for effficient lookup
             downsubid_to_subids = defaultdict(set)
-            for _, r in self._shapes.iterrows():
+            for _, r in self._routing_data.iterrows():
                 downsubid_to_subids[r.DowSubId].add(r.SubId)
 
             expanded_sub_ids = set(self._sub_ids)
@@ -323,65 +327,15 @@ class RoutingProductGridWeightImporter:
 
         # Reduce the initial dataset with the target Sub IDs
         if self._sub_ids:
-            self._shapes = self._shapes[self._shapes.SubId.isin(self._sub_ids)]
+            self._routing_data = self._routing_data[
+                self._routing_data.SubId.isin(self._sub_ids)
+            ]
 
         # -------------------------------
         # construct all grid cell polygons
         # -------------------------------
 
-        grid_cell_geom_gpd_wkt = [[[] for ilon in range(nlon)] for ilat in range(nlat)]
-
-        for ilat in range(nlat):
-
-            for ilon in range(nlon):
-
-                # -------------------------
-                # EPSG:3035   needs a swap before and after transform ...
-                # -------------------------
-                # gridcell_edges = [ [lath[ilat,ilon]    , lonh[ilat,  ilon]    ],            # for some reason need to switch lat/lon that transform works
-                #                    [lath[ilat+1,ilon]  , lonh[ilat+1,ilon]    ],
-                #                    [lath[ilat+1,ilon+1], lonh[ilat+1,ilon+1]  ],
-                #                    [lath[ilat,ilon+1]  , lonh[ilat,  ilon+1]  ]]
-
-                # tmp = self._shapes_to_geometry(gridcell_edges, epsg=crs_caea)
-                # tmp.SwapXY()              # switch lat/lon back
-                # grid_cell_geom_gpd_wkt[ilat][ilon] = tmp
-
-                # -------------------------
-                # EPSG:3573   does not need a swap after transform ... and is much faster than transform with EPSG:3035
-                # -------------------------
-                #
-                # Windows            Python 3.8.5 GDAL 3.1.3 --> lat/lon (Ming)
-                # MacOS 10.15.6      Python 3.8.5 GDAL 3.1.3 --> lat/lon (Julie)
-                # Graham             Python 3.8.2 GDAL 3.0.4 --> lat/lon (Julie)
-                # Graham             Python 3.6.3 GDAL 2.2.1 --> lon/lat (Julie)
-                # Ubuntu 18.04.2 LTS Python 3.6.8 GDAL 2.2.3 --> lon/lat (Etienne)
-                #
-                if osgeo_version < "3.0":
-                    gridcell_edges = [
-                        [
-                            lonh[ilat, ilon],
-                            lath[ilat, ilon],
-                        ],  # for some reason need to switch lat/lon that transform works
-                        [lonh[ilat + 1, ilon], lath[ilat + 1, ilon]],
-                        [lonh[ilat + 1, ilon + 1], lath[ilat + 1, ilon + 1]],
-                        [lonh[ilat, ilon + 1], lath[ilat, ilon + 1]],
-                    ]
-                else:
-                    gridcell_edges = [
-                        [
-                            lath[ilat, ilon],
-                            lonh[ilat, ilon],
-                        ],  # for some reason lat/lon order works
-                        [lath[ilat + 1, ilon], lonh[ilat + 1, ilon]],
-                        [lath[ilat + 1, ilon + 1], lonh[ilat + 1, ilon + 1]],
-                        [lath[ilat, ilon + 1], lonh[ilat, ilon + 1]],
-                    ]
-
-                tmp = self._shape_to_geometry(
-                    gridcell_edges, epsg=RoutingProductGridWeightImporter.CRS_CAEA
-                )
-                grid_cell_geom_gpd_wkt[ilat][ilon] = tmp
+        grid_cell_geom_gpd_wkt = self._compute_grid_cell_polygons()
 
         # -------------------------------
         # Derive overlay and calculate weights
@@ -389,7 +343,7 @@ class RoutingProductGridWeightImporter:
 
         grid_weights = []
 
-        for _, row in self._shapes.iterrows():
+        for _, row in self._routing_data.iterrows():
 
             poly = ogr.CreateGeometryFromWkt(row.geometry.to_wkt())
 
@@ -397,8 +351,13 @@ class RoutingProductGridWeightImporter:
             # bounding box around basin (for easy check of proximity)
             enve_basin = poly.GetEnvelope()
 
-            for ilat in range(nlat):
-                for ilon in range(nlon):
+            area_all = 0.0
+            ncells = 0
+
+            row_grid_weights = []
+
+            for ilat in range(self._nlat):
+                for ilon in range(self._nlon):
 
                     # bounding box around grid-cell (for easy check of proximity)
                     enve_gridcell = grid_cell_geom_gpd_wkt[ilat][ilon].GetEnvelope()
@@ -420,18 +379,191 @@ class RoutingProductGridWeightImporter:
 
                     area_intersect = inter.Area()
 
+                    area_all += area_intersect
+
                     if area_intersect > 0:
-                        hru_id = int(row[self._hru_id_field])
-                        cell_id = ilat * nlon + ilon
+                        hru_id = int(row[self._routing_id_field])
+                        cell_id = ilat * self._nlon + ilon
                         weight = area_intersect / area_basin
-                        grid_weights.append((hru_id, cell_id, weight))
+                        row_grid_weights.append((hru_id, cell_id, weight))
+
+            # mismatch between area of subbasin (routing product) and sum of all contributions of grid cells (model output)
+            error = (area_basin - area_all) / area_basin
+
+            if abs(error) > self._area_error_threshold and area_basin > 500000.0:
+                # record all basins with errors larger 5% (if basin is larger than 0.5 km2)
+                # error_dict[int(ibasin[key_colname])] = [error, area_basin]
+                grid_weights += row_grid_weights
+
+            else:
+                # adjust such that weights sum up to 1.0
+                for hru_id, cell_id, weight in row_grid_weights:
+                    corrected_weight = weight * 1.0 / (1.0 - error)
+                    grid_weights.append((hru_id, cell_id, corrected_weight))
+
+                # if error < 1.0:
+                #     area_all *= 1.0 / (1.0 - error)
+                # error = 0.0
 
         return GridWeightsCommand(
-            number_hrus=len(self._shapes),
-            number_grid_cells=nlon * nlat,
+            number_hrus=len(self._routing_data),
+            number_grid_cells=self._nlon * self._nlat,
             data=grid_weights,
         )
 
+    def _prepare_input_data(self):
+
+        if self._input_is_netcdf:
+
+            # Raven numbering is:
+            #
+            #      [      1      2      3   ...     1*nlon
+            #        nlon+1 nlon+2 nlon+3   ...     2*nlon
+            #           ...    ...    ...   ...     ...
+            #           ...    ...    ...   ...  nlat*nlon ]
+            #
+            # --> Making sure shape of lat/lon fields is like that
+            #
+
+            lon_var = self._input_data.variables[self._var_names[0]]
+            lon_dims = lon_var.dimensions
+            lon_var = lon_var[:]
+
+            lat_var = self._input_data.variables[self._var_names[1]]
+            lat_dims = lat_var.dimensions
+            lat_var = lat_var[:]
+
+            if len(lon_dims) == 2 and len(lat_dims) == 2:
+                if lon_dims == self._dim_names:
+                    lon_var = np.transpose(lon_var)
+
+                if lat_dims == self._dim_names:
+                    lat_var = np.transpose(lat_var)
+
+            elif len(lon_dims) == 1 and len(lat_dims) == 1:
+                # Coord vars are 1d, make them 2d
+                lon_var_size = lon_var.size
+                lon_var = np.tile(lon_var, (lat_var.size, 1))
+                lat_var = np.transpose(np.tile(lat_var, (lon_var_size, 1)))
+                assert lon_var.shape == lat_var.shape
+
+            else:
+                raise ValueError(
+                    "The coord variables of the input data must have the same number of dimensions (either 1 or 2)"
+                )
+
+            self._lath, self._lonh = self._create_gridcells_from_centers(
+                lat_var, lon_var
+            )
+
+            self._nlon = np.shape(lon_var)[1]
+            self._nlat = np.shape(lat_var)[0]
+
+        else:
+
+            # input data is a shapefile
+
+            self._input_data = self._input_data.to_crs(
+                epsg=RoutingProductGridWeightImporter.CRS_CAEA
+            )
+
+            self._nlon = 1  # only for consistency
+
+            # number of shapes in model "discretization" shapefile (not routing toolbox shapefile)
+            self._nlat = self._input_data.geometry.count()  # only for consistency
+
+    def _compute_grid_cell_polygons(self):
+
+        grid_cell_geom_gpd_wkt = [
+            [[] for ilon in range(self._nlon)] for ilat in range(self._nlat)
+        ]
+
+        if self._input_is_netcdf:
+
+            lath = self._lath
+            lonh = self._lonh
+
+            for ilat in range(self._nlat):
+
+                for ilon in range(self._nlon):
+
+                    # -------------------------
+                    # EPSG:3035   needs a swap before and after transform ...
+                    # -------------------------
+                    # gridcell_edges = [ [lath[ilat,ilon]    , lonh[ilat,  ilon]    ],            # for some reason need to switch lat/lon that transform works
+                    #                    [lath[ilat+1,ilon]  , lonh[ilat+1,ilon]    ],
+                    #                    [lath[ilat+1,ilon+1], lonh[ilat+1,ilon+1]  ],
+                    #                    [lath[ilat,ilon+1]  , lonh[ilat,  ilon+1]  ]]
+
+                    # tmp = self._routing_data_to_geometry(gridcell_edges, epsg=crs_caea)
+                    # tmp.SwapXY()              # switch lat/lon back
+                    # grid_cell_geom_gpd_wkt[ilat][ilon] = tmp
+
+                    # -------------------------
+                    # EPSG:3573   does not need a swap after transform ... and is much faster than transform with EPSG:3035
+                    # -------------------------
+                    #
+                    # Windows            Python 3.8.5 GDAL 3.1.3 --> lat/lon (Ming)
+                    # MacOS 10.15.6      Python 3.8.5 GDAL 3.1.3 --> lat/lon (Julie)
+                    # Graham             Python 3.8.2 GDAL 3.0.4 --> lat/lon (Julie)
+                    # Graham             Python 3.6.3 GDAL 2.2.1 --> lon/lat (Julie)
+                    # Ubuntu 18.04.2 LTS Python 3.6.8 GDAL 2.2.3 --> lon/lat (Etienne)
+                    #
+                    if osgeo_version < "3.0":
+                        gridcell_edges = [
+                            [
+                                lonh[ilat, ilon],
+                                lath[ilat, ilon],
+                            ],  # for some reason need to switch lat/lon that transform works
+                            [lonh[ilat + 1, ilon], lath[ilat + 1, ilon]],
+                            [lonh[ilat + 1, ilon + 1], lath[ilat + 1, ilon + 1]],
+                            [lonh[ilat, ilon + 1], lath[ilat, ilon + 1]],
+                        ]
+                    else:
+                        gridcell_edges = [
+                            [
+                                lath[ilat, ilon],
+                                lonh[ilat, ilon],
+                            ],  # for some reason lat/lon order works
+                            [lath[ilat + 1, ilon], lonh[ilat + 1, ilon]],
+                            [lath[ilat + 1, ilon + 1], lonh[ilat + 1, ilon + 1]],
+                            [lath[ilat, ilon + 1], lonh[ilat, ilon + 1]],
+                        ]
+
+                    tmp = self._shape_to_geometry(
+                        gridcell_edges, epsg=RoutingProductGridWeightImporter.CRS_CAEA
+                    )
+                    grid_cell_geom_gpd_wkt[ilat][ilon] = tmp
+
+        else:
+
+            for ishape in range(self._nlat):
+
+                idx = np.where(self._input_data[self._netcdf_input_field] == ishape)[0]
+                if len(idx) == 0:
+                    # print(
+                    #     "Polygon ID = {} not found in '{}'. Numbering of shapefile attribute '{}' needs to be [0 ... {}-1].".format(
+                    #         ishape, input_file, key_colname_model, nshapes
+                    #     )
+                    # )
+                    raise ValueError("Polygon ID not found.")
+                if len(idx) > 1:
+                    # print(
+                    #     "Polygon ID = {} found multiple times in '{}' but needs to be unique. Numbering of shapefile attribute '{}' needs to be [0 ... {}-1].".format(
+                    #         ishape, input_file, key_colname_model, nshapes
+                    #     )
+                    # )
+                    raise ValueError("Polygon ID not unique.")
+                idx = idx[0]
+                poly = self._input_data.loc[idx].geometry
+                grid_cell_geom_gpd_wkt[ishape][0] = ogr.CreateGeometryFromWkt(
+                    poly.to_wkt()
+                ).Buffer(
+                    0.0
+                )  # We add an empty buffer here to fix problems with bad polygon topology (actually caused by ESRI's historical incompetence)
+
+        return grid_cell_geom_gpd_wkt
+
     def _create_gridcells_from_centers(self, lat, lon):
 
         # create array of edges where (x,y) are always center cells
