@@ -1,44 +1,45 @@
 import datetime as dt
 
-import pytest
-
 from ravenpy.models import HMETS
+from ravenpy.utilities.testdata import get_test_data
+
+SALMON_coords = (-123.3659, 54.4848)  # (lon, lat)
+
+
+params = (
+    9.5019,
+    0.2774,
+    6.3942,
+    0.6884,
+    1.2875,
+    5.4134,
+    2.3641,
+    0.0973,
+    0.0464,
+    0.1998,
+    0.0222,
+    -1.0919,
+    2.6851,
+    0.3740,
+    1.0000,
+    0.4739,
+    0.0114,
+    0.0243,
+    0.0069,
+    310.7211,
+    916.1947,
+)
 
 
-@pytest.mark.slow
 class TestRavenERA5:
-    def test_simple(self, era5_hr):
-
-        params = (
-            9.5019,
-            0.2774,
-            6.3942,
-            0.6884,
-            1.2875,
-            5.4134,
-            2.3641,
-            0.0973,
-            0.0464,
-            0.1998,
-            0.0222,
-            -1.0919,
-            2.6851,
-            0.3740,
-            1.0000,
-            0.4739,
-            0.0114,
-            0.0243,
-            0.0069,
-            310.7211,
-            916.1947,
-        )
-
+    def test_simple(self):
+        ts = get_test_data("era5", "tas_pr_20180101-20180108.nc")[0]
         model = HMETS()
         model(
-            ts=era5_hr,
+            ts=ts,
             params=params,
             start_date=dt.datetime(2018, 1, 1),
-            end_date=dt.datetime(2018, 8, 10),
+            end_date=dt.datetime(2018, 1, 3),
             name="Salmon",
             run_name="test-hmets-era5",
             area=4250.6,
