@@ -8,10 +8,9 @@ import ravenpy
 from ravenpy.models import Ostrich, Raven
 from ravenpy.models.base import get_diff_level
 from ravenpy.tutorial import get_file
+from ravenpy.utilities.testdata import get_test_data
 
-from .common import get_test_data
-
-has_singularity = ravenpy.raven_simg.exists()
+has_singularity = False  # ravenpy.raven_simg.exists()
 
 
 class TestRaven:
