@@ -19,8 +19,9 @@ from ravenpy.models import (
     Raven,
 )
 from ravenpy.models.state import HRUStateVariables
+from ravenpy.utilities.testdata import get_test_data
 
-from .common import _convert_2d, get_test_data
+from .common import _convert_2d
 
 TS = get_test_data(
     "raven-gr4j-cemaneige", "Salmon-River-Near-Prince-George_meteo_daily.nc"
@@ -203,7 +204,6 @@ class TestGR4JCN:
 
         # Resume with final state from live model
         model_a.resume()
-        assert model_a.rvfiles["rvc"].content.startswith(":")
 
         model_a(
             TS,
@@ -215,7 +215,7 @@ class TestGR4JCN:
 
         for key in ["Soil Water[0]", "Soil Water[1]"]:
             np.testing.assert_array_almost_equal(
-                model_a.storage[1][key] - model_ab.storage[key], 0, 5
+                model_a.storage[key] - model_ab.storage[key], 0, 5
             )
 
         # Resume with final state from saved solution file
@@ -282,7 +282,7 @@ class TestGR4JCN:
             **kwargs
         )
 
-        s_b = model.storage[1]["Soil Water[0]"].isel(time=-1)
+        s_b = model.storage["Soil Water[0]"].isel(time=-1)
         assert s_a != s_b
 
     def test_update_soil_water(self):
@@ -317,8 +317,8 @@ class TestGR4JCN:
             **kwargs
         )
 
-        assert s_0 != model.storage[1]["Soil Water[0]"].isel(time=-1)
-        assert s_1 != model.storage[1]["Soil Water[1]"].isel(time=-1)
+        assert s_0 != model.storage["Soil Water[0]"].isel(time=-1)
+        assert s_1 != model.storage["Soil Water[1]"].isel(time=-1)
 
     def test_version(self):
         model = Raven()
